@@ -1,6 +1,6 @@
 /*
     Rhpc : R HPC environment
-    Copyright (C) 2012-2015  Junji NAKANO and Ei-ji Nakama
+    Copyright (C) 2012-2018  Junji NAKANO and Ei-ji Nakama
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as published by
@@ -21,6 +21,7 @@ static void Rhpc_worker_lapply_seq(int *cmd)
   int  errorOccurred=0;
   int  getcmd = 0;
   int  getsubcmd = 0;
+  int  usequote = 0;
   R_xlen_t cnti = 0;
   R_xlen_t modi = 0;
   /* data recive alloc */
@@ -47,7 +48,7 @@ static void Rhpc_worker_lapply_seq(int *cmd)
   SEXP resL;
   PROTECT_INDEX resL_ix;
 
-  GET_CMD(cmd, &getcmd, &getsubcmd, &cnti, &modi);
+  GET_CMD(cmd, &getcmd, &getsubcmd, &cnti, &modi, &usequote);
 
   /* data recive alloc */
   leni = cnti * RHPC_SPLIT_SIZE + modi;
@@ -64,9 +65,7 @@ static void Rhpc_worker_lapply_seq(int *cmd)
    }
 
   /* unserialize */
-  fun_arg=Rhpc_unserialize(data);
-  UNPROTECT(1);
-  PROTECT(fun_arg);
+  PROTECT(fun_arg=Rhpc_unserialize(data));
   PROTECT(l_fun_arg=R_NilValue);
 
   /* find function */
@@ -83,7 +82,10 @@ static void Rhpc_worker_lapply_seq(int *cmd)
   
 
   /* quote */
-  PROTECT(argq=Rhpc_enquote(arg));
+  if(usequote)
+    PROTECT(argq=Rhpc_enquote(arg));
+  else
+    PROTECT(argq=arg);
   
   /* recv X */
   {
@@ -94,6 +96,7 @@ static void Rhpc_worker_lapply_seq(int *cmd)
     R_xlen_t cntx = 0;
     R_xlen_t modx = 0;
     R_xlen_t lenx = 0;
+    int usequote=0;
     MPI_Status stat;
     SEXP datax;
     R_xlen_t msgcnt;
@@ -105,9 +108,9 @@ static void Rhpc_worker_lapply_seq(int *cmd)
     /*
       mydump((void*)cmdx,CMDLINESZ*sizeof(int));
     */
-    GET_CMD(cmdx, &getx, &getsubx, &cntx, &modx);
+    GET_CMD(cmdx, &getx, &getsubx, &cntx, &modx, &usequote);
     if( getsubx == SUBCMD_EXIT ){
-      UNPROTECT(5);
+      UNPROTECT(6);
       return;
     }
     lenx = RHPC_SPLIT_SIZE * cntx + modx;
@@ -175,7 +178,10 @@ static void Rhpc_worker_lapply_seq(int *cmd)
       if(i)
 	SET_VECTOR_ELT(argw,i,VECTOR_ELT(argq,i-1));
       else
-	SET_VECTOR_ELT(argw,i,LCONS(install("quote"),CONS(X,R_NilValue)));
+	if (usequote)
+	  SET_VECTOR_ELT(argw,i,LCONS(install("quote"),CONS(X,R_NilValue)));
+	else
+	  SET_VECTOR_ELT(argw,i,X);
       if(!isNull(names)){
 	SET_STRING_ELT(namesymbol,i,(i)?STRING_ELT(names, i-1):mkChar(""));
       }else{
@@ -187,8 +193,11 @@ static void Rhpc_worker_lapply_seq(int *cmd)
 
     /* eval */
     errorOccurred=0;
-    PROTECT(lng = LCONS(Rhpc_docall, CONS(fun,CONS(argw, R_NilValue))));
-    ret=R_tryEval(lng, R_GlobalEnv, &errorOccurred);
+    if(usequote)
+      PROTECT(lng = LCONS(Rhpc_docall, CONS(fun,CONS(argw, R_NilValue))));
+    else
+      PROTECT(lng = LCONS(install("do.call"), CONS(fun, CONS(argw, R_NilValue))));
+    PROTECT(ret=R_tryEval(lng, R_GlobalEnv, &errorOccurred));
     DPRINT("errorOccurred=%d\n",errorOccurred);
 
 
@@ -215,7 +224,7 @@ static void Rhpc_worker_lapply_seq(int *cmd)
     /* PROTECT(ret); */
     SET_VECTOR_ELT(resL,works,ret);
     REPROTECT(resL,resL_ix);
-    UNPROTECT(5);
+    UNPROTECT(6);
   }
  
   DPRINT("success eval x\n");
@@ -223,7 +232,7 @@ static void Rhpc_worker_lapply_seq(int *cmd)
   /* serialize */
   {
     PROTECT(l_out=R_NilValue);
-    PROTECT(out=Rhpc_serialize(resL));
+    PROTECT(out=Rhpc_serialize_norealloc(resL));
     DPRINT("success  serialize outlength=%ld\n",xlength(out));
   }
 
@@ -240,7 +249,7 @@ static void Rhpc_worker_lapply_seq(int *cmd)
     MPI_Status  *status  = Calloc(reqcnt+1, MPI_Status);
     int calls;
 
-    SET_CMD(cmdo, CMD_NAME_LAPPLY_SEQ, SUBCMD_NORMAL, cnto, modo );
+    SET_CMD(cmdo, CMD_NAME_LAPPLY_SEQ, SUBCMD_NORMAL, cnto, modo, usequote );
 
     calls=0;
     _M(MPI_Isend(cmdo,   (int)CMDLINESZ,
@@ -266,7 +275,7 @@ static void Rhpc_worker_lapply_seq(int *cmd)
     Free(status);
     DPRINT("send data wait end\n");
   }
-  UNPROTECT(10);
+  UNPROTECT(11);
 
   /* recv exit */
   {
@@ -282,7 +291,7 @@ static void Rhpc_worker_lapply_seq(int *cmd)
       mydump((void*)cmdx,CMDLINESZ*sizeof(int));
     */
 
-    GET_CMD(cmdx, &getx, &getsubx, &cntx, &modx);
+    GET_CMD(cmdx, &getx, &getsubx, &cntx, &modx, &usequote);
     if( getsubx == SUBCMD_EXIT ){
       return;
     }
